# Contribute to Yeti

Yeti is built by the YUI team at Yahoo and contributors like you. We are excited
that you are interested in improving Yeti with us. This guide will help make sure
your contribution will be ready to ship.

If you are considering larger changes to Yeti, please discuss your proposed change
on the [Yeti Forums][forums] to coordinate effort and get feedback from Yeti users.
For small changes and bugfixes, opening a bug (if it does not already exist) and
posting that you're working on it is just fine. Thank you!

## Getting Started

 1. Make sure you have a [YUILibrary.com][yui] account.
    - We use this to track Yeti bugs and contributor CLAs.
    - Sign up with the same e-mail address you use in your Git commit messages.
    - Using the same address allows us to verify your [CLA][] is on file.
 1. Make sure you have a [GitHub.com][gh] account.
 1. [Submit a ticket][newticket] for your issue, if it does not already exist.
    - Check [Yeti's open issues][issues] to see if your issue was already reported.
 1. Post a comment on the issue's ticket noting that you plan to work on it.
 1. Fork the [Yeti repository][repo] on GitHub.

## Making Changes

 1. Run `npm test` before making changes to make sure your setup is working.
 1. Create a topic branch for your contribution.
    - This is usually based off master.
      You can create a topic with `git checkout -b my-bugfix-branch-name`
 1. Make commits that describe a single logical change. See the example below.
 1. Run `git diff --check` and remove unnecessary whitespace.
 1. Run `make lint` and make sure your new code runs through the linter
    without error.
 1. Run `make coverage` and make sure your new code is covered with a test.
    Tests are located in `test` and use [Vows][].
 1. Make sure all existing tests pass.

### Example Commit Message

    Provide example. 50 chars, present-tense. Fix #1.

    Detailed explanatory text, if necessary. Be clear and concise.
    Wrap text to about 72 characters. The blank line between the
    summary and body is important; Git tools expect the blank line
    and may not behave correctly if it's omitted. Sometimes the first
    line is treated as the subject of the email and the rest as the body.

    Write your commit message in the present tense: "Fix foo" and not "Fixed
    foo." This convention matches the commit messages generated by tools
    like git merge and git revert.

    Include the YUILibrary.com bug number in your summary, e.g. "Fix #1."

    Further paragraphs come after blank lines.

    - You can also create lists with bullet points.

    - A hyphen is used for the bullet, preceded by a single space, with
      blank lines in between.

    - Use a hanging indent.

## Submitting Changes

 1. Sign the [YUI Contributor License Agreement][CLA].
 1. Push your changes to a topic branch in your fork of the repository.
 1. Submit a pull request to the repository in the yui organization.
 1. Update your YUILibrary.com ticket with the URL of your pull request.

## Get Help

 - [README.md](https://github.com/yui/yeti/blob/master/README.md)
 - [Yeti Forums][forums]
 - #yui on [irc.freenode.net](http://webchat.freenode.net/)

Thanks for working to make Yeti better. We really, really appreciate it!

  [forums]: https://groups.google.com/forum/#!categories/yui-support/tools
  [CLA]: http://yuilibrary.com/contribute/cla/
  [yui]: http://yuilibrary.com/
  [gh]: https://github.com/
  [issues]: https://github.com/yui/yeti/issues?state=open
  [newticket]: https://github.com/yui/yeti/issues/new
  [repo]: https://github.com/yui/yeti
  [Vows]: http://vowsjs.org/
